title: Radical Redesign or Incremental Change?
url: http://www.nngroup.com/articles/radical-incremental-redesign/
hash_url: 3c22950431d673a3fbea2ba140392ac7

<p>The biggest user-experience question is not a matter of any individual design issue, such as whether to use <a href="http://www.nngroup.com/articles/mega-menus-work-well/">mega menus</a> or some other form of <a href="http://www.nngroup.com/articles/ia-questions-navigation-menus/">navigation menu</a>. The bigger picture requires you to first decide on the overriding UX strategy: to shoot for a big bang and change everything in one go, or to rely in incremental quality improvements, one step at a time.</p>
<p>Since <a href="/articles/quality-assurance-ux/">user experience is a quality discipline</a>, there is much to be said for <a href="/articles/fresh-vs-familiar-aggressive-redesign/">incremental design changes</a>, since we know from extensive QA research that <a href="/articles/usability-and-six-sigma/">continuous improvement</a> is known to lead to high-quality products. On the other hand, there’s also much to be said for improving everything at once and potentially achieve a much higher quality boost than would result from any one incremental quality tweak.</p>
<p>So how to pick a UX strategy: one big revolution, or many small steps?</p>
<h2>Website Overhaul for the Right Reasons</h2>
<p>Site redesigns often require a tremendous amount of coordination and resources. Sometimes, a redesign project can be a purely visual reskinning of the entire site, with new styles, layouts, and treatments. Other times, serious taxonomy, information architecture, content, or usability issues are being addressed. Either way, make sure that your redesign project is based on user data and has clear goals and measures of success. Make incremental changes to reach these goals and only resort to a complete redesign if the data tells you to do so.</p>
<p>Examine your site’s <a href="http://www.nngroup.com/articles/conversion-rates/">conversion rates</a> and <a href="http://www.nngroup.com/articles/which-ux-research-methods/">get customer feedback</a> to determine what the issues are and whether a legitimate reason exists to take on a major project. Gathering the right data reduces the risk of delivering a design that users don’t want or that of compromising the return on your investment.</p>
<p><strong>Don’t let panic or boredom lead you astray.</strong> When embarking on a site-redesign project, companies sometimes fall prey to “the grass is always greener” syndrome. That is, they believe that other people (or companies) have it better, even though this belief can be completely false.</p>
<p>When organizations ask me to help them with a redesign project, one of my first questions is WHY—why do you want to redesign? Frequently the response is, “We haven’t updated it in many years.” Or, “It looks old and amateurish.” The unspoken reason for many redesign projects is, “I’m bored with it.”</p>
<p><em>You</em> may be bored with your current site, but <em>customers</em> likely aren’t: they usually don’t sit and stare at the site for extended periods every day. Most companies are lucky if customers visit their site once per month, and even in the case of more frequent visits, users tend to <em>like</em> designs that are safe and familiar.</p>
<p>Before you throw out the old and bring in the new, make sure you have solid evidence that doing so is necessary to achieve user-centered goals. Discuss solutions that address the root of the problem. Too often, designers approach solving problems by recommending trendy changes while ignoring less-glamorous aspects such as <a href="http://www.nngroup.com/articles/write-interesting-facts/">content</a>, <a href="http://www.nngroup.com/articles/ia-vs-navigation/">structure</a>, or interaction design, which are often the source of the problems. Visitors don’t spend nearly as much time as you do staring at your own design, so they don’t have that burning desire for what’s in fashion. Customers are extremely task-oriented and care much more about a usable website than a pretty one.</p>
<h2>Usually Choose Incremental Changes Over a Major Overhaul</h2>
<p>Drastic website changes are jarring for users and risky for business. The cost and effort of getting an entire organization and senior stakeholders to agree on the new website is enormous. Organizations often launch a redesign project with an ambitious timeline, only to see it get stretched and stretched by endless debate. Some projects start and stop multiple times, while other projects fizzle completely. Teams start out excited, but soon become miserable.</p>
<p>The most successful redesign projects have clear, measurable objectives. Before you commit to a full site redesign, you need to define the problems you want to solve and the results you wish to obtain.</p>
<p><strong>Never make radical changes when minimal adjustments will suffice.</strong> Too many websites undergo a major overhaul unnecessarily. While legitimate reasons exist for engaging in a redesign, the reality is that many problems you need to solve are isolated and can be fixed with smaller, incremental approaches. <a href="http://www.nngroup.com/articles/analytics-reports-ux-strategists/">Look at the metrics</a> rather than guess what needs to be changed. Allow data to help you determine the extent of the problem and apply the least amount of change necessary to solve it. Radical changes have a higher chance of inadvertently breaking something critical for users.</p>
<p><strong>Be aware of your own cognitive biases.</strong> The human mind works in ways that cloud objectivity. The old saying goes, “if all you have is a hammer, everything looks like a nail.” The tools and technology available to website makers can divert their attention from the true issues and fog their judgment in taking the appropriate course of action. Visual designers may rely on visceral responses in justifying a redesign, and may be biased towards aesthetics-driven solutions. Good designers are aware of these biases and allow data to guide them in selecting the right solutions.</p>
<p><strong>Consider the cost of switching users to a new interface. </strong><a href="http://www.nngroup.com/articles/fresh-vs-familiar-aggressive-redesign/">People don’t like change.</a> Don’t be lured into thinking that a new site will attract customers. To reduce cognitive effort humans make assumptions on how to interact with interfaces based on previous experiences. Once people learn a way of doing things they expect the system to work the same way in the future.</p>
<p>Avoid disrupting the user experience with wild design changes without careful consideration. Customers balk at change, even when the new design is clearly better. Assess the impact of change on users before you decide to make the switch. Spending 1 or 2 days conducting a <a href="http://www.nngroup.com/articles/redesign-competitive-testing/">usability study of your existing</a> site is invaluable.</p>
<h2>Sometimes a Major Overhaul Is Best</h2>
<p>Extensive changes are risky and generally should be avoided. However, good reasons exist for a major site overhaul at rare occasions. If the issues go beyond aesthetics or the problems are glaring and no longer can be remedied with band-aid fixes, sometimes it’s best to rip off the bandage and fix the problems. Below are some reasons for taking the plunge:</p>
<ul>
<li><strong>The gains from making incremental changes are miniscule or nonexistent</strong>: For years you’ve attempted gradual fixes but now there’s not much more you can do to improve the site. You’ve performed many iterations and found yourself at the point of diminishing returns.</li>
<li><strong>The technology is severely outdated, making critical changes impossible: </strong>The backend system can’t support interactions and features necessary for critical user journeys. Customers can’t access your content well on mobile devices and social features are clunky. Third-party tools that worked well several years ago can’t keep up with the changing needs of your customers.</li>
<li><strong>Architecturally the site is a tangled mess</strong>: Your site is a big mess and doesn’t function the way visitors expect. The company brand strategy has changed significantly over the years. You’ve altered the site’s<a href="http://www.nngroup.com/articles/ia-vs-navigation/"> navigation structure incrementally to accommodate changes in the information architecture</a> and shifting strategies. However, after several years of patchwork the site is incohesive and people can’t accomplish what they need.</li>
<li><strong>Severely low conversion rates site-wide</strong>: Data analysis reveals a bounty of issues that are impossible to fix to customers’ satisfaction. <a href="http://www.nngroup.com/articles/conversion-rates/">Conversion measurements</a> indicate extremely high exit rates and <a href="http://www.nngroup.com/articles/reduce-bounce-rates/">bounce rates</a>, and few page views. The issues are so tangled and enormous that no amount of incremental fixes can increase conversions to an acceptable level.</li>
</ul>
<ul>
<li><strong><a href="http://www.nngroup.com/articles/competitive-usability-evaluations/">Benchmarking research</a> reveals your site is far inferior to the competition: </strong>You don’t want to change your site every time your competitors do. However, you must do so if research shows that you’re losing customers because other sites are doing a better job of supporting user needs.</li>
</ul>
<h2>Conclusion</h2>
<p>The path you choose (incremental vs. major overhaul) should align user needs with business goals. Leverage information from conversion research (i.e., site analytics, user testing) to help you identify the issues and determine how to solve them. Solid numbers keep you focused on the right issues and prevent political arguments.</p>
